@@ -5,7 +5,7 @@ use strict;
 use Time::Piece;
 use Digest::HMAC_SHA1;
 use MIME::Base64 qw(encode_base64 decode_base64);
-use XML::MyXML 0.052 qw(tidy_xml simple_to_xml xml_to_object);
+use XML::MyXML 0.098061 qw(tidy_xml simple_to_xml xml_to_object);
 use SOAP::MySOAP 0.023;
 use Carp;
 use Data::Dumper;
@@ -16,11 +16,11 @@ SOAP::Amazon::S3 - A module for interfacing with Amazon S3 through SOAP
 
 =head1 VERSION
 
-Version 0.0231
+Version 0.0401
 
 =cut
 
-our $VERSION = '0.0231';
+our $VERSION = '0.0401';
 
 =head1 SYNOPSIS
 
@@ -136,10 +136,13 @@ EOB
 	if ($self->{'RaiseError'} and $self->{'error'}) {
 		print "\nAmazon returned Fault:\n";
 		print Dumper($self->{'error'}->simplify);
+		$obj->delete();
 		confess;
 	}
 
-	return $self->{'soaper'}->{'response'}->content;
+	my $ret = $self->{'soaper'}->{'response'}->content;
+	$obj->delete();
+	return $ret if defined wantarray;
 }
 
 =head1 OBJECT METHODS
@@ -155,6 +158,7 @@ sub listbuckets {
 	my $xml = $self->ListAllMyBuckets;
 	my $obj = &xml_to_object($xml);
 	my @buckets = map {$_->simplify} $obj->path("Body/ListAllMyBucketsResponse/ListAllMyBucketsResponse/Buckets/Bucket");
+	$obj->delete();
 	foreach my $bucket (@buckets) {
 		$bucket->{'_s3'} = $self;
 		$bucket->{'Name'} = $bucket->{'Bucket'}{'Name'};
@@ -234,6 +238,7 @@ sub list {
 	my $xml = $s3->ListBucket(Bucket => $self->{'Name'});
 	my $obj = &XML::MyXML::xml_to_object($xml);
 	my @objects = map {values %{$_->simplify}} $obj->path('Body/ListBucketResponse/ListBucketResponse/Contents');
+	$obj->delete();
 	foreach my $object (@objects) {
 		$object->{'_s3'} = $s3;
 		$object->{'Bucket'} = $self->{'Name'};
@@ -255,9 +260,9 @@ sub name {
 	return $self->{'Name'};
 }
 
-=head2 $bucket->putobject( $obj_key, $obj_data, { 'Content-Type' => $mime_type } )
+=head2 $bucket->putobject( $obj_key, $obj_data, { 'Content-Type' => $mime_type, metadata => {color=>'red', feel=>'soft'} } )
 
-Creates an object in the S3 bucket, named $obj_key. The {} section is optional, and may contain the Content-Type (defaults to 'text/plain'). Returns an ...::S3::Object type object pointing to the object just created, if successful.
+Creates an object in the S3 bucket, named $obj_key. The {} section is optional, and may contain the Content-Type (defaults to 'text/plain') and a metadata hashref. Returns an ...::S3::Object type object pointing to the object just created, if successful.
 
 =cut
 
@@ -267,10 +272,12 @@ sub putobject {
 	my $data = shift;
 	my $options = shift || {};
 
-	my $mimetype = $options->{'Content-Type'} || 'text/plain';
+	my $metadata = $options->{'metadata'} || {};
+	$metadata->{'Content-Type'} ||= $options->{'Content-Type'} || 'text/plain';
+	$metadata = [ map {(MetaData => [Name => $_, Value => $metadata->{$_}])} keys %$metadata ];
 
 	my $s3 = $self->{'_s3'};
-	$s3->PutObjectInline( Bucket => $self->name, Key => $key, Metadata => [ Name => 'Content-Type', Value => $mimetype ], Data => MIME::Base64::encode_base64($data), ContentLength => length($data) );
+	$s3->PutObjectInline( Bucket => $self->name, Key => $key, @$metadata, Data => MIME::Base64::encode_base64($data), ContentLength => length($data) );
 	bless { _s3 => $s3, Bucket => $self->name, _bucket => $self, Key => $key }, 'SOAP::Amazon::S3::Object' unless $s3->{'error'};
 }
 
@@ -343,8 +350,9 @@ sub acl {
 		my @grants = $xml->path('Body/GetObjectAccessControlPolicyResponse/GetObjectAccessControlPolicyResponse/AccessControlList/Grant');
 		foreach my $grant (@grants) {
 			my $uri = $grant->path('Grantee/URI');
-			if ($uri and $uri->value eq 'http://acs.amazonaws.com/groups/global/AllUsers') { return 'public'; }
+			if ($uri and $uri->value eq 'http://acs.amazonaws.com/groups/global/AllUsers') { $xml->delete(); return 'public'; }
 		}
+		$xml->delete();
 		return 'private';
 	}
 }
@@ -365,10 +373,22 @@ sub getdata {
 	return if $s3->{'error'};
 	my $obj = &XML::MyXML::xml_to_object($resp);
 	my $data = $obj->path('Body/GetObjectResponse/GetObjectResponse/Data');
-	if ($data) { $data = $data->value; } else { return; }
+	if ($data) { $data = $data->value; } else { $obj->delete(); return; }
+	$obj->delete();
 	return MIME::Base64::decode_base64($data);
 }
 
+=head2 $object->url
+
+Return the URL of the object
+
+=cut
+
+sub url {
+	my $self = shift;
+
+	return "http://s3.amazonaws.com/".$self->{'_bucket'}->name()."/".$self->name();
+}
 
 =head1 AUTHOR
 
@@ -408,13 +428,17 @@ L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=SOAP-Amazon-S3>
 
 L<http://search.cpan.org/dist/SOAP-Amazon-S3>
 
+=item * Module's RSS feed
+
+L<http://myperl.eu/permodule/SOAP-Amazon-S3>
+
 =back
 
 =head1 ACKNOWLEDGEMENTS
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2006 Alexander Karelas, all rights reserved.
+Copyright 2006-2007 Alexander Karelas, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
